# coding: koi8-r
# ====================================================
#	Cycle - calendar for women
#	Distributed under GNU Public License
# Author: Oleg S. Gints (altgo@users.sourceforge.net)
# Home page: http://cycle.sourceforge.net
# ===================================================
import hashlib
import pickle
import os.path
import os
import cal_year
import wx
import warnings
# deprecated since release 2.3
warnings.filterwarnings("ignore",
                        category=DeprecationWarning,
                        message='.*rotor module', module=__name__)

try:
    import rotor
except:
    import p_rotor as rotor


def Save_Cycle(name='cycle', passwd='123', file='cycle'):
    """ Save the contents of our document to disk.
    """
    objSave = []
    m = hashlib.md5()
    m.update(passwd.encode('utf-8'))
    rt = rotor.newrotor(m.digest())
    objSave.append(['period', cal_year.cycle.period])
    objSave.append(['by_average', cal_year.cycle.by_average])
    objSave.append(['disp', cal_year.cycle.disp])
    objSave.append(['first_week_day', cal_year.cycle.first_week_day])
    objSave.append(['note', cal_year.cycle.note])
    for d in cal_year.cycle.begin:
        objSave.append(['begin', [d.GetDay(), d.GetMonth(), d.GetYear()]])

    for d in cal_year.cycle.last:
        objSave.append(['last', [d.GetDay(), d.GetMonth(), d.GetYear()]])

    for d in cal_year.cycle.tablet:
        objSave.append(['tablet', [d.GetDay(), d.GetMonth(), d.GetYear()]])

    for d in list(cal_year.cycle.colour_set.keys()):
        objSave.append(['colour', [d, cal_year.cycle.colour_set[d].Get()]])

    tmp = rt.encrypt(b'Cycle'+pickle.dumps(objSave))
    tmp = b"UserName="+pickle.dumps(name)+b"==="+tmp
    p, f_name = get_f_name(file)

    if not os.path.exists(p):
        os.mkdir(p, 0o700)
    f = open(f_name, "wb")
    f.write(tmp)
    f.close()
#    print "All data saved to disk"


def Load_Cycle(name='cycle', passwd='123', file='cycle'):

    p, f_name = get_f_name(file)
    if os.path.isfile(f_name):
        m = hashlib.md5()
        m.update(passwd.encode('utf-8'))
        rt = rotor.newrotor(m.digest())
        f = open(f_name, "rb")
        tmp = f.read()
        if tmp[:len(b"UserName=")] == b"UserName=":
            # new format
            n = tmp.find(b"===")+len(b"===")
            tmp = tmp[n:]  # remove username
        tmp = rt.decrypt(tmp)
        f.close()
        if tmp[0:5] != b'Cycle':
            #	    print 'Password is invalid'
            return False
        else:
            tmp = tmp[5:]  # remove control word 'Cycle'
            objLoad = pickle.loads(tmp)
            set_color_default()
            for type, d in objLoad:
                #		print "Load: ", type, d
                if type == 'period':
                    cal_year.cycle.period = int(d)
                elif type == 'by_average':
                    cal_year.cycle.by_average = int(d)
                elif type == 'disp':
                    cal_year.cycle.disp = int(d)
                elif type == 'first_week_day':
                    cal_year.cycle.first_week_day = int(d)
                elif type == 'begin':
                    dt = wx.DateTime.FromDMY(d[0], d[1], d[2])
                    cal_year.cycle.begin.append(dt)
                elif type == 'last':
                    dt = wx.DateTime.FromDMY(d[0], d[1], d[2])
                    cal_year.cycle.last.append(dt)
                elif type == 'tablet':
                    dt = wx.DateTime.FromDMY(d[0], d[1], d[2])
                    cal_year.cycle.tablet.append(dt)
                elif type == 'note':
                    cal_year.cycle.note = d.copy()
                elif type == 'colour':  # d=['item', (r,g,b)]
                    c = wx.Colour(d[1][0], d[1][1], d[1][2])
                    if d[0] in cal_year.cycle.colour_set:
                        cal_year.cycle.colour_set[d[0]] = c
                    else:
                        cal_yaar.cycle.colour_set.update({d[0]: c})
#	    print "Load OK"
            return True

# -------------------------------------------------------


def get_f_name(name=""):
    if '__WXMSW__' in wx.PlatformInfo:
        p = os.path.join(os.getcwd(), "data")
    else:
        p = os.path.expanduser("~/.cycle")
    f_name = os.path.join(p, name)

    return p, f_name

# -------------------------------------------------------


def set_color_default():
    cal_year.cycle.colour_set = {'begin': wx.TheColourDatabase.Find('RED'),
                                 'prog begin': wx.TheColourDatabase.Find('PINK'),
                                 'conception': wx.TheColourDatabase.Find('MAGENTA'),
                                 'safe sex': wx.TheColourDatabase.Find('WHEAT'),
                                 'fertile': wx.TheColourDatabase.Find('GREEN YELLOW'),
                                 'ovule': wx.TheColourDatabase.Find('SPRING GREEN'),
                                 '1-st tablet': wx.TheColourDatabase.Find('GOLD'),
                                 'pause': wx.TheColourDatabase.Find('LIGHT BLUE'),
                                 'next 1-st tablet': wx.TheColourDatabase.Find('PINK')}



"""This module was written by Gerd Woetzel:
   http://mail.python.org/pipermail/python-list/2005-January/261304.html


   This module is derived from Modules/rotormodule.c and translated
   into Python. I have appended the Copyright by Lance Ellinghouse
   below. The rotor module has been removed from the Python 2.4
   distribution because

      the rotor module uses an insecure algorithm and is deprecated.
      ==============================================================

   Of course, this does still hold. However, I think this module might
   be used and adapted for demonstration purposes and might help some
   poor users who have encrypted (or obfuscated) some old stuff with
   the rotor module and have no access to older Python versions any
   more.

   Documentation can be found in

   Python Library Reference,  Guido van Rossum, Fred L. Drake, Jr., editor,
   PythonLabs, Release 2.3.4 May 20, 2004
   <http://www.python.org/doc/2.3.4/lib/module-rotor.html>

   #####################################################################
   Copyright 1994 by Lance Ellinghouse,
   Cathedral City, California Republic, United States of America.

                        All Rights Reserved

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted,
   provided that the above copyright notice appear in all copies and that
   both that copyright notice and this permission notice appear in
   supporting documentation, and that the name of Lance Ellinghouse
   not be used in advertising or publicity pertaining to distribution
   of the software without specific, written prior permission.

   LANCE ELLINGHOUSE DISCLAIMS ALL WARRANTIES WITH REGARD TO
   THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS, IN NO EVENT SHALL LANCE ELLINGHOUSE BE LIABLE FOR ANY SPECIAL,
   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
   NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
   #####################################################################
"""


class newrotor(object):

    def __init__(self, key, n_rotors=6):
        self.n_rotors = n_rotors
        self.setkey(key)

    def setkey(self, key):
        self.key = key
        self.rotors = None
        self.positions = [None, None]

    def encrypt(self, buf):
        # Reset (encrypt) positions and encryptmore
        self.positions[0] = None
        return self.cryptmore(buf, 0)

    def encryptmore(self, buf):
        return self.cryptmore(buf, 0)

    def decrypt(self, buf):
        # Reset decrypt positions and decryptmore
        self.positions[1] = None
        return self.cryptmore(buf, 1)

    def decryptmore(self, buf):
        return self.cryptmore(buf, 1)

    def cryptmore(self, buf, do_decrypt):
        size, nr, rotors, pos = self.get_rotors(do_decrypt)
        outbuf = []
        append = outbuf.append
        for c in buf:
            if do_decrypt:
                # Apply decrypt rotors and xor in reverse order
                for i in range(nr-1, -1, -1):
                    c = pos[i] ^ rotors[i][c]
            else:
                # Apply xor and ecrypt rotors
                for i in range(nr):
                    c = rotors[i][c ^ pos[i]]
            append(c)

            # Advance rotors, i.e. add the (fixed) rotor increments to the
            # variable rotor positions with carry.
            # Note:  In the C-implementation, the result of the carry addition
            #        was stored to an "unsigned char". Hence the next carry
            #        is lost if pos[i] == size-1 and pnew >= size.
            #        Masking with 0xff simulates this behavior.
            #
            pnew = 0  # (pnew >= size) works as "carry bit"
            for i in range(nr):
                pnew = ((pos[i] + (pnew >= size)) & 0xff) + rotors[i][size]
                pos[i] = pnew % size

        return bytes(outbuf)

    def get_rotors(self, do_decrypt):
        # Return a tuple (size, nr, rotors, positions) where
        # - size is the rotor size (== 256, because of 8-bit bytes)
        # - nr is the number of rotors.
        # - rotors is a tuple of nr encrypt or nr decrypt rotors
        #     for do_decrypt == 0 or do_decrypt == 1 respectively.
        # - postions is a list of nr "rotor positions".
        #
        # The rotors represent the static aspect of the rotor machine which
        # is initially computed from key and fixed during en/decryption.
        # A rotor is a random permutation of range(size) extended
        # by an "increment value" in range(size).
        #
        # The followng statements hold for a tuple of encrypt rotors E and
        # and the corresponding tuple of decrypt rotors D.
        #
        #      D[i][E[i][j]] == j for i in range(nr) for j in range(size)
        #
        #      E[i][D[i][j]] == j for i in range(nr) for j in range(size)
        #
        #      This means that the corresponding rotors E[i] and D[i] are
        #      inverse permutations.
        #      The increments are equal for the corresponding encrypt and
        #      decrypt rotors and have an odd value:
        #
        #      D[i][size] == E[i][size] and E[i][size] == 1 mod 2 and
        #          0 < E[i][size] < size for i in range(nr)
        #
        # The position vector represents the dynamic aspect.
        # It changes after each en/decrypted character (the rotors
        # are "advanced"). The initial position vector is also computed
        # from the key
        #
        nr = self.n_rotors
        rotors = self.rotors
        positions = self.positions[do_decrypt]

        if positions is None:
            if rotors:
                positions = list(rotors[3])
            else:
                # Generate identity permutation for 8-bit bytes plus an
                # (unused) increment value
                self.size = size = 256
                id_rotor = list(range(size+1))

                # Generate nr "random" initial positions and "random"
                # en/decrypt rotors from id_rotor.
                #
                rand = random_func(self.key)
                E = []
                D = []
                positions = []
                for i in range(nr):
                    i = size
                    positions.append(rand(i))
                    erotor = id_rotor[:]
                    drotor = id_rotor[:]
                    drotor[i] = erotor[i] = 1 + 2*rand(i//2)  # increment
                    while i > 1:
                        r = rand(i)
                        i -= 1
                        er = erotor[r]
                        erotor[r] = erotor[i]
                        erotor[i] = er
                        drotor[er] = i
                    drotor[erotor[0]] = 0
                    E.append(tuple(erotor))
                    D.append(tuple(drotor))
                self.rotors = rotors = (
                    tuple(E), tuple(D), size, tuple(positions))
            self.positions[do_decrypt] = positions
        return rotors[2], nr, rotors[do_decrypt], positions


def random_func(key):
    # Generate a random number generator that is "seeded" from key.
    # This algorithm is copied from Python2.3 randommodule.c.
    #
    mask = 0xffff
    x = 995
    y = 576
    z = 767
    for c in key:
        x = (((x << 3 | x >> 13) + c) & mask)
        y = (((y << 3 | y >> 13) ^ c) & mask)
        z = (((z << 3 | z >> 13) - c) & mask)

    # Emulate (unintended?) cast to short
    maxpos = mask >> 1
    mask += 1
    if x > maxpos:
        x -= mask
    if y > maxpos:
        y -= mask
    if z > maxpos:
        z -= mask

    y |= 1  # avoid very bad seed, why not for x and z too?

    # Oh, dear, for compatibility, we must evaluate the first seed transition
    # the hard way, later it becomes much simpler
    x = 171 * (x % 177) - 2 * (x//177)
    y = 172 * (y % 176) - 35 * (y//176)
    z = 170 * (z % 178) - 63 * (z//178)
    if x < 0:
        x += 30269
    if y < 0:
        y += 30307
    if z < 0:
        z += 30323
    # At least all values are > 0 by now but may be greater than expected ..

    def rand(n, seed=[(x, y, z)]):
        # Return a random number 0 <= r < n
        #
        x, y, z = seed[0]
        seed[0] = ((171*x) % 30269, (172*y) % 30307, (170*z) % 30323)
        return int((x/30269.0 + y/30307.0 + z/30323.0) * n) % n

        # Original code was like this:
        #    from math import floor
        #    val = x/30269.0 + y/30307.0 + z/30323.0
        #    val = val - floor(val)
        #    if val >= 1.0:
        #       val = 0.0
        #    n = int(val*n) % n

    return rand



# coding: koi8-r
# ====================================================
#	Cycle - calendar for women
#	Distributed under GNU Public License
# Author: Oleg S. Gints (altgo@users.sourceforge.net)
# Home page: http://cycle.sourceforge.net
# ===================================================
from cal_year import cycle, Val
from set_dir import *
from save_load import Load_Cycle, get_f_name, set_color_default
import pickle
import wx.html
import wx
import os
import random, string
import warnings
# deprecated since release 2.3
warnings.filterwarnings("ignore",
                        category=RuntimeWarning,
                        message='.*tempnam is a potential security risk to your program', module=__name__)

# ---------------------------------------------------------------------------


class Settings_Dlg(wx.Dialog):
    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, -1, _('Settings'), wx.DefaultPosition)
        self.Centre(wx.BOTH)
        # ======================
        box = wx.BoxSizer(wx.VERTICAL)

        b1 = wx.StaticBoxSizer(wx.StaticBox(
            self, -1, _('Length of cycle')), wx.VERTICAL)
        i = wx.NewId()
        self.cb1 = wx.CheckBox(self, i, _(' by average'), style=wx.NO_BORDER)
        b1.Add(self.cb1, 0, wx.ALL, 5)
        self.Bind(wx.EVT_CHECKBOX, self.By_Average, id=i)
        self.cb1.SetValue(cycle.by_average)

        b2 = wx.BoxSizer(wx.HORIZONTAL)
        i = wx.NewId()
        self.sc = wx.SpinCtrl(self, i, "", size=wx.Size(50, -1))
        self.sc.SetRange(21, 35)
        self.sc.SetValue(cycle.period)
        self.sc.Enable(not self.cb1.GetValue())
        b2.Add(self.sc, 0)
        b2.Add(wx.StaticText(self, -1, _(' days in cycle')), 0)
        b1.Add(b2, 0, wx.ALL, 5)
        box.Add(b1, 0, wx.EXPAND | wx.TOP | wx.LEFT | wx.RIGHT, 10)
        # ======================
        self.rb = wx.RadioBox(self, -1, _('Display'),
                              choices=[_('safe sex days'), _(
                                  'fertile days'), _('both')],
                              majorDimension=1, style=wx.RA_SPECIFY_COLS)
        box.Add(self.rb, 0, wx.EXPAND | wx.TOP | wx.LEFT | wx.RIGHT, 10)
        self.rb.SetSelection(cycle.disp)

        # ======================
        self.rb1 = wx.RadioBox(self, -1, _('First week day'),
                               choices=[_('monday'), _('sunday')],
                               majorDimension=1, style=wx.RA_SPECIFY_COLS)
        box.Add(self.rb1, 0, wx.EXPAND | wx.ALL, 10)
        self.rb1.SetSelection(cycle.first_week_day)

        # ======================
        i = wx.NewId()
        txt1 = _('Colours')
        txt2 = _('Change password')
        w1, h = self.GetTextExtent(txt1)
        w2, h = self.GetTextExtent(txt2)
        w = max(w1, w2)
        box.Add(wx.Button(self, i, txt1, size=wx.Size(w+10, -1)),
                0, wx.ALIGN_CENTER)
        self.Bind(wx.EVT_BUTTON, self.OnColours, id=i)
        # ======================
        i = wx.NewId()
        box.Add(wx.Button(self, i, txt2, size=wx.Size(w+10, -1)),
                0, wx.TOP | wx.ALIGN_CENTER, 10)
        self.Bind(wx.EVT_BUTTON, self.OnChangePasswd, id=i)

        # ======================
        but_box = wx.BoxSizer(wx.HORIZONTAL)
        i = wx.NewId()
        but_box.Add(wx.Button(self, i, _('Ok')), 0, wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnOk, id=i)

        i = wx.NewId()
        but_box.Add(wx.Button(self, i, _('Cancel')), 0, wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnCancel, id=i)

        box.Add(but_box, 0, wx.ALIGN_CENTER)

        self.SetAutoLayout(True)
        self.SetSizer(box)
        box.Fit(self)

    def By_Average(self, event):
        if event.Checked():
            self.sc.Enable(False)
        else:
            self.sc.Enable(True)

    def OnOk(self, event):
        if not 21 <= self.sc.GetValue() <= 35:
            dlg = wx.MessageDialog(self, _('Period of cycle is invalid!'),
                                   _('Error!'), wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return
        cycle.period = self.sc.GetValue()
        cycle.by_average = self.cb1.GetValue()
        cycle.disp = self.rb.GetSelection()
        cycle.first_week_day = self.rb1.GetSelection()
        self.EndModal(wx.ID_OK)

    def OnCancel(self, event):
        self.EndModal(wx.ID_CANCEL)

    def OnChangePasswd(self, event):
        dlg = Ask_Passwd_Dlg(self)
        dlg.ShowModal()
        dlg.Destroy()

    def OnColours(self, event):
        dlg = Colours_Dlg(self)
        dlg.ShowModal()
        dlg.Destroy()
# ---------------------------------------------------------------------------


class Ask_Passwd_Dlg(wx.Dialog):
    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, -1, _('Password'))

        # ======================
        box = wx.BoxSizer(wx.VERTICAL)

        box.Add(wx.StaticText(self, -1, _('Enter your password')), 0,
                wx.ALIGN_CENTER | wx.TOP | wx.LEFT | wx.RIGHT, 10)
        self.pass1 = wx.TextCtrl(self, -1, "", wx.Point(10, 30),
                                 size=(130, -1), style=wx.TE_PASSWORD)
        box.Add(self.pass1, 0, wx.ALIGN_CENTER | wx.ALL, 10)

        box.Add(wx.StaticText(self, -1, _('Once more...')), 0,
                wx.ALIGN_CENTER | wx.LEFT | wx.RIGHT, 10)
        self.pass2 = wx.TextCtrl(self, -1, "", wx.Point(10, 80),
                                 size=(130, -1), style=wx.TE_PASSWORD)
        box.Add(self.pass2, 0, wx.ALIGN_CENTER | wx.ALL, 10)

        b1 = wx.BoxSizer(wx.HORIZONTAL)
        i = wx.NewId()
        b1.Add(wx.Button(self, i, _('Ok')), 0, wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnOk, id=i)

        i = wx.NewId()
        b1.Add(wx.Button(self, i, _('Cancel')), 0, wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnCancel, id=i)
        self.pass1.SetFocus()
        box.Add(b1, 0, wx.ALIGN_CENTER)

        self.SetAutoLayout(True)
        self.SetSizer(box)
        box.Fit(self)

    def OnOk(self, event):
        err = ""
        if self.pass1.GetValue() == "" or self.pass2.GetValue() == "":
            err = _('Password must be not EMPTY!')
        if self.pass1.GetValue() != self.pass2.GetValue():
            err = _('Entering password don\'t match!')
        if err != "":
            dlg = wx.MessageDialog(self, err,
                                   _('Error!'), wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return
        cycle.passwd = self.pass1.GetValue()
        self.EndModal(wx.ID_OK)

    def OnCancel(self, event):
        self.EndModal(wx.ID_CANCEL)


# ---------------------------------------------------------------------------
def get_users():
    # Get list of users
    magic_str = b"UserName="
    users = []  # array of (user, file) name
    p, f_name = get_f_name()
    if os.path.exists(p):
        files = os.listdir(p)
        for f in files:
            fd = open(os.path.join(p, f), "rb")
            tmp = fd.read(len(magic_str))
            if tmp == magic_str:
                tmp = fd.read(100)
                n = tmp.find(b"===")  # find end string
                if n != -1:
                    users.append((pickle.loads(tmp[:n]), f))
                else:  # old format, user_name=file_name
                    users.append((f, f))
        # if not users:
        # users=[(_('empty'),"empty")]
        users.sort()
    return users

# ---------------------------------------------------------------------------


class Login_Dlg(wx.Dialog):
    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, -1, _('Login'))

        self.name = ""
        self.file = ""

        box = wx.BoxSizer(wx.VERTICAL)

        # Get list of users
        self.users = get_users()

#	p, f_name=get_f_name()
#	if os.path.exists(p):
#	    users=os.listdir(p)
#	else:
#	    users=[_('empty')]
#   	users.sort()

        # ======== List users ==============
        i = wx.NewId()
        self.il = wx.ImageList(16, 16, True)
        bmp = wx.Bitmap(os.path.join(
            bitmaps_dir, 'smiles.bmp'), wx.BITMAP_TYPE_BMP)
        mask = wx.Mask(bmp, wx.WHITE)
        bmp.SetMask(mask)

        idx1 = self.il.Add(bmp)

        self.list = wx.ListCtrl(self, i, size=wx.Size(200, 200),
                                style=wx.LC_REPORT | wx.SUNKEN_BORDER | wx.LC_SINGLE_SEL)
        self.list.SetImageList(self.il, wx.IMAGE_LIST_SMALL)
        self.list.InsertColumn(0, _('Your name'))
        for k in range(len(self.users)):
            self.list.InsertItem(k, self.users[k][0], idx1)
        self.list.SetColumnWidth(0, 180)
        self.list.SetItemState(0, wx.LIST_STATE_SELECTED,
                               wx.LIST_STATE_SELECTED)
        self.name = self.users[0][0]
        self.file = self.users[0][1]

        self.list.Bind(wx.EVT_LIST_ITEM_SELECTED,
                       self.OnItemSelected, self.list)
        self.list.Bind(wx.EVT_LIST_KEY_DOWN, self.OnKeyDown, self.list)

        box.Add(self.list, 0, wx.ALL, 10)

        # ========= Add user =============
        i = wx.NewId()
        box.Add(wx.Button(self, i, _('Add user')), 0, wx.ALIGN_CENTER)
        self.Bind(wx.EVT_BUTTON, self.OnAdd, id=i)

        # ========= Ok - Cancel =============
        b1 = wx.BoxSizer(wx.HORIZONTAL)
        i = wx.NewId()
        b1.Add(wx.Button(self, i, _('Ok')), 0, wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnOk, id=i)

        i = wx.NewId()
        b1.Add(wx.Button(self, i, _('Cancel')), 0, wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnCancel, id=i)
        box.Add(b1, 0, wx.ALIGN_CENTER)

        self.SetAutoLayout(True)
        self.SetSizer(box)
        box.Fit(self)
        self.list.SetFocus()

    def OnItemSelected(self, event):
        # self.list.GetItemText(event.GetIndex())
        self.name = self.users[event.GetIndex()][0]
        self.file = self.users[event.GetIndex()][1]

    def OnKeyDown(self, event):
        if event.GetKeyCode() == ord(" ") or event.GetKeyCode() == wx.WXK_RETURN:
            self.OnOk()
        else:
            event.Skip()

    def OnAdd(self, event=None):
        if ask_name(self):
            self.EndModal(wx.ID_OK)

    def OnOk(self, event=None):
        dlg = wx.TextEntryDialog(self, self.name+_(', enter you password:'), _('Password'), '',
                                 style=wx.OK | wx.CANCEL | wx.TE_PASSWORD)
        while dlg.ShowModal() == wx.ID_OK:
            cycle.passwd = dlg.GetValue()
            cycle.name = self.name
            cycle.file = self.file
            if Load_Cycle(cycle.name, cycle.passwd, cycle.file):
                dlg.Destroy()
                self.EndModal(wx.ID_OK)
                return
            else:
                dlg2 = wx.MessageDialog(self, _('Password is invalid!'),
                                        _('Error!'), wx.OK | wx.ICON_ERROR)
                dlg2.ShowModal()
                dlg2.Destroy()
        dlg.Destroy()

    def OnCancel(self, event):
        self.EndModal(wx.ID_CANCEL)

# -------------------------------------------------------


def first_login():
    # Get list of users
    users = get_users()
    if users != []:
        return 'not_first'  # user(s) already exists
    if ask_name():
        return 'first'
    else:
        return 'bad_login'
# -------------------------------------------------------

def randomword(length):
   letters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
   return ''.join(random.choice(letters) for i in range(length))

def get_new_file_name():
    while True:
        f = "cycle"+randomword(6)
        p, f_name = get_f_name(f)
        if not os.path.isfile(f_name):
            break
    return f
# -------------------------------------------------------


def ask_name(parent=None):
    # nobody, it is first login
    wx.MessageBox(
        _("This program is not a reliable contraceptive method.\n"
          "Neither does it help to prevent sexually transmitted diseases\n"
          "like HIV/AIDS.\n\nIt is just an electronic means of keeping track\n"
          "of some of your medical data and extracting some statistical\n"
          "conclusions from them. You cannot consider this program as a\n"
          "substitute for your gynecologist in any way."))
    dlg = wx.TextEntryDialog(parent, _('Enter your name:'), _('New user'), '',
                             style=wx.OK | wx.CANCEL)
    while dlg.ShowModal() == wx.ID_OK:
        name = dlg.GetValue()
        if name != "":
            users = get_users()
            exists = False
            for i in users:
                if name == i[0]:
                    exists = True
                    break
            if not exists:
                d = Ask_Passwd_Dlg(parent)
                if d.ShowModal() == wx.ID_OK:
                    cycle.file = get_new_file_name()
                    cycle.name = name
                    d.Destroy()
                    dlg.Destroy()
                    # self.EndModal(wx.ID_OK)
                    set_color_default()
                    return True
                else:
                    d.Destroy()
                    continue
            else:
                err = name+_(' - already exists!')
        else:
            err = _('Name must be not EMPTY')
        d2 = wx.MessageDialog(dlg, err,	_('Error!'), wx.OK | wx.ICON_ERROR)
        d2.ShowModal()
        d2.Destroy()

    dlg.Destroy()
    return False

# ---------------------------------------------------------------------------


class Legend_Dlg(wx.Dialog):
    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, -1, _('Legend'))

        # ======================
        box = wx.BoxSizer(wx.VERTICAL)

        self._add(box, _('today'), wx.NullColour, wx.SIMPLE_BORDER)
        self._add(box, _('begin of cycle'), cycle.colour_set['begin'])
        self._add(box, _('prognosis of cycle begin'),
                  cycle.colour_set['prog begin'])
        self._add(box, _('conception'), cycle.colour_set['conception'])
        self._add(box, _('safe sex'), cycle.colour_set['safe sex'])
        self._add(box, _('fertile'), cycle.colour_set['fertile'])
        self._add(box, _('ovulation, birth'), cycle.colour_set['ovule'])
        self._add(box, _('1-st tablet'), cycle.colour_set['1-st tablet'])
        self._add(box, _('tablets no. 22-28 or pause'),
                  cycle.colour_set['pause'])
        self._add(box, _('next 1-st tablet'),
                  cycle.colour_set['next 1-st tablet'])

        i = wx.NewId()
        box.Add(wx.Button(self, i, _('Ok')), 0, wx.ALIGN_CENTER | wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnOk, id=i)

        self.SetAutoLayout(True)
        self.SetSizer(box)
        box.Fit(self)

    def _add(self, box, txt, col, st=0):
        b = wx.BoxSizer(wx.HORIZONTAL)
        w = wx.Window(self, -1, size=wx.Size(15, 15), style=st)
        w.SetBackgroundColour(col)
        b.Add(w, 0, wx.LEFT | wx.RIGHT, 10)
        b.Add(wx.StaticText(self, -1, txt), 0,
              wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 10)
        box.Add(b, 0, wx.TOP, 10)

    def OnOk(self, event):
        self.EndModal(wx.ID_OK)

# ---------------------------------------------------------------------------


class Note_Dlg(wx.Dialog):
    def __init__(self, parent, title="", txt=""):
        wx.Dialog.__init__(self, parent, -1, title)
        self.CentreOnParent(wx.BOTH)

        # ======================
        box = wx.BoxSizer(wx.VERTICAL)
        self.txt = wx.TextCtrl(self, -1, txt,
                               size=(-1, 100), style=wx.TE_MULTILINE)
        box.Add(self.txt, 0,
                wx.EXPAND | wx.ALIGN_CENTER | wx.TOP | wx.LEFT | wx.RIGHT, 10)

        b1 = wx.BoxSizer(wx.HORIZONTAL)
        i = wx.NewId()
        b1.Add(wx.Button(self, i, _('Ok')), 0, wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnOk, id=i)

        i = wx.NewId()
        b1.Add(wx.Button(self, i, _('Cancel')), 0, wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnCancel, id=i)

        i = wx.NewId()
        b1.Add(wx.Button(self, i, _('Remove')), 0, wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnRemove, id=i)
        box.Add(b1, 0, wx.ALIGN_CENTER)

        self.SetAutoLayout(True)
        self.SetSizer(box)
        box.Fit(self)
        self.txt.SetFocus()

    def OnOk(self, event):
        self.EndModal(wx.ID_OK)

    def OnCancel(self, event):
        self.EndModal(wx.ID_CANCEL)

    def OnRemove(self, event):
        self.EndModal(False)

    def Get_Txt(self):
        return self.txt.GetValue()

# ---------------------------------------------------------------------------


class MyHtmlWindow(wx.html.HtmlWindow):
    def __init__(self, parent, id, pos=wx.DefaultPosition, size=wx.DefaultSize):
        wx.html.HtmlWindow.__init__(self, parent, id, pos, size)
        if "gtk2" in wx.PlatformInfo:
            self.SetStandardFonts()

    def OnLinkClicked(self, linkinfo):
        pass

# ---------------------------------------------------------------------------


class Help_Dlg(wx.Dialog):
    def __init__(self, parent, title="", txt=""):
        wx.Dialog.__init__(self, parent, -1, title)
        self.CentreOnParent(wx.BOTH)

        # ======================
        box = wx.BoxSizer(wx.VERTICAL)
        self.html = MyHtmlWindow(self, -1, size=(500, 350))
        self.html.SetPage(txt)
        box.Add(self.html, 0, wx.ALIGN_CENTER |
                wx.TOP | wx.LEFT | wx.RIGHT, 10)

        i = wx.NewId()
        box.Add(wx.Button(self, i, _('Ok')), 0, wx.ALIGN_CENTER | wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnOk, id=i)

        self.SetAutoLayout(True)
        self.SetSizer(box)
        box.Fit(self)

    def OnOk(self, event):
        self.EndModal(wx.ID_OK)

# ---------------------------------------------------------------------------


class Colours_Dlg(wx.Dialog):
    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, -1, _('Colours settings'))

        self.col_set = cycle.colour_set.copy()
        self.col_id = list(cycle.colour_set.keys())
        self.data = wx.ColourData()
        self.data.SetChooseFull(True)
        self.buttons = {}
        # ======================
        box = wx.BoxSizer(wx.VERTICAL)

        self._add(box, _('begin of cycle'), 'begin')
        self._add(box, _('prognosis of cycle begin'), 'prog begin')
        self._add(box, _('conception'), 'conception')
        self._add(box, _('safe sex'), 'safe sex')
        self._add(box, _('fertile'), 'fertile')
        self._add(box, _('ovulation, birth'), 'ovule')
        self._add(box, _('1-st tablet'), '1-st tablet')
        self._add(box, _('tablets no. 22-28 or pause'), 'pause')
        self._add(box, _('next 1-st tablet'), 'next 1-st tablet')

        b1 = wx.BoxSizer(wx.HORIZONTAL)
        i = wx.NewId()
        b1.Add(wx.Button(self, i, _('Ok')), 0, wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnOk, id=i)

        i = wx.NewId()
        b1.Add(wx.Button(self, i, _('By default')), 0, wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnDefault, id=i)

        i = wx.NewId()
        b1.Add(wx.Button(self, i, _('Cancel')), 0, wx.ALL, 10)
        self.Bind(wx.EVT_BUTTON, self.OnCancel, id=i)
        box.Add(b1, 0, wx.ALIGN_CENTER)

        self.SetAutoLayout(True)
        self.SetSizer(box)
        box.Fit(self)

    def _add(self, box, txt, col):
        b = wx.BoxSizer(wx.HORIZONTAL)
        i = self.col_id.index(col)
        bt = wx.Button(self, i, "", size=wx.Size(15, 15))
        self.Bind(wx.EVT_BUTTON, self.get_colour, id=i)
        bt.SetBackgroundColour(self.col_set[col])
        self.buttons.update({i: bt})
        b.Add(bt, 0, wx.LEFT | wx.RIGHT, 10)
        b.Add(wx.StaticText(self, -1, txt), 0,
              wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 10)
        box.Add(b, 0, wx.TOP, 10)

    def get_colour(self, event):
        c = self.col_set[self.col_id[event.GetId()]]
        self.data.SetColour(c)
        dlg = wx.ColourDialog(self, self.data)
        if dlg.ShowModal() == wx.ID_OK:
            self.data = dlg.GetColourData()
            c = self.data.GetColour()
            self.buttons[event.GetId()].SetBackgroundColour(c)
            self.col_set[self.col_id[event.GetId()]] = c

    def OnOk(self, event):
        cycle.colour_set = self.col_set.copy()
        Val.Cal.Draw_Mark()
        self.EndModal(wx.ID_OK)

    def OnDefault(self, event):
        self.col_set = {'begin': wx.NamedColour('RED'),
                        'prog begin': wx.NamedColour('PINK'),
                        'conception': wx.NamedColour('MAGENTA'),
                        'safe sex': wx.NamedColour('WHEAT'),
                        'fertile': wx.NamedColour('GREEN YELLOW'),
                        'ovule': wx.NamedColour('SPRING GREEN'),
                        '1-st tablet': wx.NamedColour('GOLD'),
                        'pause': wx.NamedColour('LIGHT BLUE'),
                        'next 1-st tablet': wx.NamedColour('PINK')}
        for item in self.col_id:
            self.buttons[self.col_id.index(item)].SetBackgroundColour(
                self.col_set[item])

    def OnCancel(self, event):
        self.EndModal(wx.ID_CANCEL)

# ---------------------------------------------------------------------------



#!/usr/bin/env python3
# coding: koi8-r
# ====================================================
#	Cycle - calendar for women
#	Distributed under GNU Public License
# Author: Oleg S. Gints (altgo@users.sourceforge.net)
# Home page: http://cycle.sourceforge.net
# ===================================================
import builtins
import gettext
from set_dir import *
from dialogs import *
from save_load import *
from cal_year import *
import wx.lib.colourdb
import wx.html
import wx
import os
import sys
import gettext
import locale

#from prn import *

lang_find = False
if not '__WXMSW__' in wx.PlatformInfo:
    for lang_env_var in ('LANGUAGE', 'LC_ALL', 'LC_CTYPE', 'LANG'):
        if lang_find:
            break
        if lang_env_var in os.environ:
            env_language = os.environ[lang_env_var]
            for s_lang in env_language.split(':'):  # if set more languages
                os.environ[lang_env_var] = s_lang
                try:
                    dl = locale.getdefaultlocale()
                    lang = [dl[0][0:2]]
                    l = gettext.translation('cycle', msg_dir, lang)
                    if wx.USE_UNICODE:
                        builtins.__dict__['_'] = lambda s: l.ugettext(s)
                    else:
                        builtins.__dict__['_'] = lambda s: l.ugettext(
                            s).encode(dl[1])
                    _('try decode this string')
                    lang_find = True
                    break  # language was found
                except:
                    pass
else:  # for MS Windows
    try:
        dl = locale.getdefaultlocale()
        lang = [dl[0][0:2]]
        l = gettext.translation('cycle', msg_dir, lang)
        if wx.USE_UNICODE:
            builtins.__dict__['_'] = lambda s: l.ugettext(s)
        else:
            builtins.__dict__['_'] = lambda s: l.ugettext(s).encode(dl[1])
        _('try decode this string')
        lang_find = True
    except:
        pass

if not lang_find:
    builtins.__dict__['_'] = lambda s: s
    lang = [""]


class MyFrame(wx.Frame):
    def __init__(self, parent, ID, title):
        wx.Frame.__init__(self, parent, ID, title,
                          wx.DefaultPosition, wx.Size(800, 600))

#	self.printer = wx.HtmlEasyPrinting()
        icon = wx.Icon(os.path.join(
            icons_dir, 'mini/cycle.xpm'), wx.BITMAP_TYPE_XPM)
        self.SetIcon(icon)

        Val.frame = self
        self.CreateStatusBar()
        self.MakeToolMenu()  # toolbar

        self.cal = Cal_Year(self)
        self.OnCurrent(self)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

    def OnCloseWindow(self, event):
        Save_Cycle(cycle.name, cycle.passwd, cycle.file)
        self.Destroy()

    def TimeToQuit(self, event):
        self.Close(True)

    def MakeToolMenu(self):
        tb = self.CreateToolBar(wx.TB_HORIZONTAL | wx.NO_BORDER)
        tb.SetToolBitmapSize(wx.Size(24, 24))

        qDec=SetToolPath(self, tb, 10, os.path.join(
                bitmaps_dir, 'dec.png'), _('Dec Year'))
        self.Bind(wx.EVT_TOOL, self.OnDecYear, qDec)

        qCurr=SetToolPath(self, tb, 20, os.path.join(
                bitmaps_dir, 'curr.png'), _('Current Year'))
        self.Bind(wx.EVT_TOOL, self.OnCurrent, qCurr)

        qInc=SetToolPath(self, tb, 30, os.path.join(
                bitmaps_dir, 'inc.png'), _('Inc Year'))
        self.Bind(wx.EVT_TOOL, self.OnIncYear, qInc)

        tb.SetToolSeparation(50)
        tb.AddSeparator()

        qLeg=SetToolPath(self, tb, 40, os.path.join(
                bitmaps_dir, 'legend.png'), _('Legend'))
        self.Bind(wx.EVT_TOOL, self.Legend, qLeg)

        qExp=SetToolPath(self, tb, 45, os.path.join(
                bitmaps_dir, 'export.png'), _('Export to iCal'))
        self.Bind(wx.EVT_TOOL, self.Export, qExp)

        qSett=SetToolPath(self, tb, 50, os.path.join(
                bitmaps_dir, 'set.png'), _('Settings'))
        self.Bind(wx.EVT_TOOL, self.Settings, qSett)

        qHelp=SetToolPath(self, tb, 55, os.path.join(
                bitmaps_dir, 'help.png'), _('Help'))
        self.Bind(wx.EVT_TOOL, self.Info, qHelp)

#	SetToolPath(self, tb, 57, os.path.join(bitmaps_dir,'help.png'), _('Print'))
#	wx.EVT_TOOL(self, 57, self.test)

        tb.AddSeparator()

        qExit=SetToolPath(self, tb, 60, os.path.join(
                bitmaps_dir, 'exit.png'), _('Exit'))
        self.Bind(wx.EVT_TOOL, self.TimeToQuit, qExit)

        tb.Realize()

    def test(self, event):
        #rpt = report_year(self.cal.year)
        # self.printer.PreviewText(rpt)
        # self.printer.PreviewFile('2.html')
        dlg = Colour_Dlg(self)
        dlg.ShowModal()
        dlg.Destroy()

    def Legend(self, event):
        dlg = Legend_Dlg(self)
        dlg.ShowModal()
        dlg.Destroy()

    def Export(self, event):
        dlg = wx.FileDialog(self, _("Export to iCal"),
                            style=wx.FD_SAVE)

        if dlg.ShowModal() == wx.ID_OK:
            try:
                fileobj = file(dlg.GetPath(), "w")
                report_year_ical(self.cal.year, fileobj)
            except (IOError, OSError) as err:
                wx.MessageDialog(
                    self, str(err), _("Unable to export"), style=wx.OK).ShowModal()

    def Settings(self, event):
        dlg = Settings_Dlg(self)
        if dlg.ShowModal() == wx.ID_OK:
            self.cal.Set_Year(wx.DateTime.Today().GetYear())
        dlg.Destroy()

    def Info(self, event):
        global lang
        f_name = os.path.join(doc_dir, "README_"+lang[0]+".html")
        if not os.path.isfile(f_name):
            f_name = os.path.join(doc_dir, "README.html")
        f = open(f_name, "r")
        msg = f.read()
        dlg = Help_Dlg(self, _('Help'), msg)
        dlg.ShowModal()

    # increment and decrement toolbar controls

    def OnIncYear(self, event):
        self.cal.Inc_Year()

    def OnDecYear(self, event):
        self.cal.Dec_Year()

    def OnCurrent(self, event):
        self.cal.Set_Year(wx.DateTime.Today().GetYear())


# ----------------------------------------------
def SetToolPath(self, tb, id, bmp, title):
    global dir_path
#    tb.AddSimpleTool(id, wx.Bitmap(os.path.join(dir_path, bmp), wx.BITMAP_TYPE_PNG),
#                     title, title)
    return tb.AddTool(id, "", wx.Bitmap(os.path.join(dir_path, bmp), wx.BITMAP_TYPE_PNG),
                     wx.NullBitmap, wx.ITEM_NORMAL, title, title)


class MyApp(wx.App):
    def OnInit(self):
        wx.lib.colourdb.updateColourDB()
        ret = first_login()
        if ret == 'bad_login':
            return True
        elif ret == 'not_first':
            dlg = Login_Dlg(None)
            if dlg.ShowModal() == wx.ID_CANCEL:
                dlg.Destroy()
                return True
            dlg.Destroy()
        self.frame_init()
        return True

    def frame_init(self):
        frame = MyFrame(None, -1, "")
        frame.Show(True)
        self.SetTopWindow(frame)


if __name__ == '__main__':
    locale.setlocale(locale.LC_ALL, "")
    dir_path = os.getcwd()
    app = MyApp(0)
    app.MainLoop()



#!/usr/bin/env python

from distutils.core import setup

setup(name="cycle",
      version="0.3.1",
      description="Calendar for women",
      author="Oleg Gints",
      author_email="altgo@users.sourceforge.net",
      url="http://cycle.sourceforge.net",
      py_modules=['cycle'],
      )



# coding: koi8-r
# ====================================================
#	Cycle - calendar for women
#	Distributed under GNU Public License
# Author: Oleg S. Gints (altgo@users.sourceforge.net)
# Home page: http://cycle.sourceforge.net
# ===================================================

import wx
import wx.adv
import calendar
import operator
from functools import reduce


class Val:
    pass


MARK_BEGIN = 1
MARK_FERT = 1 << 1
MARK_OVUL = 1 << 2
MARK_SAFESEX = 1 << 3
MARK_TODAY = 1 << 4
MARK_NOTE = 1 << 5
MARK_PROG = 1 << 6
MARK_LAST = 1 << 7  # last cycle, conception begin
MARK_BIRTH = 1 << 8
MARK_TABLET = 1 << 9  # 1-st hormonal tablet
MARK_T22_28 = 1 << 10  # tablets 22-28 or pause 7 days
MARK_NEXT_TABLET = 1 << 11

# -------------------- class Month_Cal -------------------


class Month_Cal(wx.adv.GenericCalendarCtrl):
    def __init__(self, parent, id, dt, pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=0):

        style = wx.adv.CAL_NO_MONTH_CHANGE | wx.NO_BORDER
        if cycle.first_week_day == 0:
            style = style | wx.adv.CAL_MONDAY_FIRST
        else:
            style = style | wx.adv.CAL_SUNDAY_FIRST
        try:
            style = style | wx.adv.CAL_SEQUENTIAL_MONTH_SELECTION
        except NameError:
            pass

        wx.adv.GenericCalendarCtrl.__init__(
            self, parent, id, dt, pos, size, style)
        self.SetBackgroundColour(wx.WHITE)
        self.SetHeaderColours(wx.BLACK, wx.WHITE)
        if '__WXMSW__' in wx.PlatformInfo:
            font = self.GetFont()
            font.SetFaceName("MS Sans Serif")
            self.SetFont(font)

        self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_KEY_UP, self.OnKey)
        self.Bind(wx.EVT_KEY_DOWN, self.OnKey)
        self.d_click = wx.DateTime()  # FromDMY(1, 0,2002)

    def OnLeftDown(self, event):
        # HitTest(Point pos) -> (result, date, weekday)
        res, d, w = self.HitTest(event.GetPosition())
        if res == wx.adv.CAL_HITTEST_DAY:
            Val.frame.SetStatusText(info(d))

    def OnRightDown(self, event):
        res, d, w = self.HitTest(event.GetPosition())
        if res == wx.adv.CAL_HITTEST_DAY:
            # now d contain clicked day
            self.d_click = d
            menu = wx.Menu()
            menu.Append(1, d.Format('%d %B'))
            menu.AppendSeparator()
            menu.AppendCheckItem(2, _('Beginning of cycle'))
            menu.Check(2, is_set_mark(d, MARK_BEGIN, d.GetYear()))
            menu.AppendCheckItem(5, _('1-st tablet'))
            menu.Check(5, is_set_mark(d, MARK_TABLET, d.GetYear()))
            if is_set_mark(d, MARK_BEGIN, d.GetYear()):
                menu.AppendCheckItem(3, _('Conception'))
                menu.Check(3, is_set_mark(d, MARK_LAST, d.GetYear()))
            menu.AppendCheckItem(4, _('Note'))
            menu.Check(4, is_set_mark(d, MARK_NOTE, d.GetYear()))

            self.Bind(wx.EVT_MENU, self.OnBegin, id=2)
            self.Bind(wx.EVT_MENU, self.OnLast, id=3)
            self.Bind(wx.EVT_MENU, self.OnNote, id=4)
            self.Bind(wx.EVT_MENU, self.OnTablet, id=5)
            self.PopupMenu(menu, event.GetPosition())
            menu.Destroy()

    def OnBegin(self, event):
        if self.d_click in cycle.begin:
            cycle.begin.remove(self.d_click)
            if self.d_click in cycle.last:
                cycle.last.remove(self.d_click)
            remove_mark(self.d_click, MARK_BEGIN, self.d_click.GetYear())
            remove_mark(self.d_click, MARK_LAST, self.d_click.GetYear())
        else:
            cycle.begin.append(self.d_click)
            cycle.begin.sort()
            add_mark(self.d_click, MARK_BEGIN, self.d_click.GetYear())
        Val.Cal.Draw_Mark()

    def OnLast(self, event):
        if self.d_click in cycle.begin:
            if self.d_click in cycle.last:
                cycle.last.remove(self.d_click)
                remove_mark(self.d_click, MARK_LAST, self.d_click.GetYear())
            else:
                cycle.last.append(self.d_click)
                cycle.last.sort()
                add_mark(self.d_click, MARK_LAST, self.d_click.GetYear())
            Val.Cal.Draw_Mark()

    def OnNote(self, event):
        txt = get_note(self.d_click)
        dlg = Note_Dlg(self, self.d_click.Format('%d %B'), txt)
        ret = dlg.ShowModal()
        t = dlg.Get_Txt()
        dlg.Destroy()
        if ret == wx.ID_OK:
            add_note(self.d_click, t)
            add_mark(self.d_click, MARK_NOTE, self.d_click.GetYear())
        elif ret == False:
            remove_note(self.d_click)
            remove_mark(self.d_click, MARK_NOTE, self.d_click.GetYear())
        elif ret == wx.ID_CANCEL:
            return
        Val.Cal.Draw_Mark()

    def OnTablet(self, event):
        if self.d_click in cycle.tablet:
            cycle.tablet.remove(self.d_click)
            remove_mark(self.d_click, MARK_TABLET, self.d_click.GetYear())
        else:
            cycle.tablet.append(self.d_click)
            cycle.tablet.sort()
            add_mark(self.d_click, MARK_TABLET, self.d_click.GetYear())
        Val.Cal.Draw_Mark()

    def OnKey(self, event):
        k = event.GetKeyCode()
        if k == wx.WXK_LEFT or k == wx.WXK_RIGHT or \
           k == wx.WXK_UP or k == wx.WXK_DOWN:
            pass
        else:
            event.Skip()


# -------------------- class Cal_Year -------------------
class Cal_Year(wx.ScrolledWindow):
    def __init__(self, parent):
        wx.ScrolledWindow.__init__(self, parent, -1)
#	self.SetScrollbars(20, 20, 39, 40)
        self.SetBackgroundColour('LIGHT BLUE')

        dt = wx.DateTime.Today()
        self.year = dt.GetYear()

        self.day_of_year = []
        self.month = []
        Val.Cal = self
        self.Init_Year()
#	self.Draw_Mark()

    def Inc_Year(self):
        self.year += 1
        self.Draw_Year()
        reset_mark(self.year)
        self.Draw_Mark()

    def Dec_Year(self):
        self.year -= 1
        self.Draw_Year()
        reset_mark(self.year)
        self.Draw_Mark()

    def Set_Year(self, year):
        self.year = year
        self.Draw_Year()
        reset_mark(self.year)
        self.Draw_Mark()

    def Init_Year(self):
        m = 0
        box = wx.BoxSizer(wx.VERTICAL)
        box.Add(wx.StaticLine(self, -1, style=wx.LI_HORIZONTAL), 0, wx.EXPAND)
        for y in range(3):
            row_box = wx.BoxSizer(wx.HORIZONTAL)
            for x in range(4):
                t = wx.DateTime.FromDMY(1, m, self.year)
                id = wx.NewId()
                self.month.append(Month_Cal(self, id, t, ))
                row_box.Add(self.month[m], 0, wx.ALL, 5)
                m += 1
            box.Add(row_box, 0, wx.LEFT | wx.RIGHT, 10)

        self.SetAutoLayout(True)
        self.SetSizer(box)
        box.Fit(self)
        w = box.GetSize().GetWidth()
        h = box.GetSize().GetHeight()
        Val.frame.SetClientSize(wx.Size(w+10, h+90))
        self.SetScrollbars(20, 20, w/20, h/20)

    def Draw_Year(self):
        Val.frame.SetTitle(cycle.name+" - "+str(self.year))
        for m in range(12):
            t = wx.DateTime.FromDMY(1, m, self.year)
            self.month[m].EnableMonthChange(True)
            self.month[m].SetDate(t)
            self.month[m].EnableMonthChange(False)
            self.month[m].Refresh()

    def Draw_Mark(self):

        f_norm = self.month[1].GetFont()
        f_norm.SetUnderlined(False)
        f_ul = self.month[1].GetFont()
        f_ul.SetUnderlined(True)

        dt = wx.DateTime.Today()
        if self.year == dt.GetYear():
            add_mark(dt, MARK_TODAY, self.year)

        calc_fert(self.year)
        calc_tablet(self.year)
        k = 1
        for m in range(12):
            sel_hide = True  # need hide selection
            for d in range(1, wx.DateTime.GetNumberOfDays(m, self.year)+1):
                can_hide = True
                lab = cycle.mark.get(k, 0)
                at = wx.adv.CalendarDateAttr(wx.BLACK)
                at.SetBackgroundColour(wx.WHITE)
                at.SetFont(f_norm)

                dt = wx.DateTime.FromDMY(d, m, self.year)
                if not dt.IsWorkDay():  # mark weekend
                    at.SetTextColour('firebrick')

                if lab & MARK_BEGIN:
                    at.SetBackgroundColour(cycle.colour_set['begin'])
                    at.SetTextColour(wx.WHITE)

                if lab & MARK_PROG:
                    at.SetBackgroundColour(cycle.colour_set['prog begin'])
                    at.SetTextColour(wx.BLACK)

                if lab & MARK_SAFESEX and (cycle.disp == 0 or cycle.disp == 2):
                    at.SetBackgroundColour(cycle.colour_set['safe sex'])

                if lab & MARK_FERT and (cycle.disp == 1 or cycle.disp == 2):
                    at.SetBackgroundColour(cycle.colour_set['fertile'])

                if lab & MARK_OVUL and (cycle.disp == 1 or cycle.disp == 2):
                    at.SetBackgroundColour(cycle.colour_set['ovule'])

                if lab & MARK_TODAY:
                    at.SetBorderColour('NAVY')
                    at.SetBorder(wx.adv.CAL_BORDER_SQUARE)
                    can_hide = False

                if lab & MARK_LAST:
                    at.SetBackgroundColour(cycle.colour_set['conception'])

                if lab & MARK_NOTE:
                    at.SetFont(f_ul)
                    can_hide = False

                if lab & MARK_BIRTH:
                    at.SetBackgroundColour(cycle.colour_set['ovule'])

                if lab & MARK_TABLET:
                    at.SetBackgroundColour(cycle.colour_set['1-st tablet'])

                if lab & MARK_T22_28:
                    at.SetBackgroundColour(cycle.colour_set['pause'])

                if lab & MARK_NEXT_TABLET:
                    at.SetBackgroundColour(
                        cycle.colour_set['next 1-st tablet'])

                if sel_hide and can_hide:
                    # we can hide selection when don't use border and underlined
                    sel_hide = False
                    self.month[m].SetDate(dt)
                    self.month[m].SetHighlightColours(at.GetTextColour(),
                                                      at.GetBackgroundColour())

                self.month[m].SetAttr(d, at)

                k += 1

        # so visual refresh is more fast
        for m in range(12):
            self.month[m].Refresh()


# -------------------- work with cycle -------------------
class cycle:
    begin = []
    last = []
    tablet = []
    period = 28
    mark = {}
    passwd = "123"
    name = "empty"
    file = "empty"
    by_average = False
    disp = 0
    first_week_day = 0
    note = {}
    colour_set = {}
#    colour_set={'begin':wx.NamedColour('red'),
#	'prog begin':wx.NamedColour('pink'),
#        'conception':wx.NamedColour('MAGENTA'),
#	'safe sex':wx.NamedColour('wheat'),
#	'fertile':wx.NamedColour('green yellow'),
#	'ovule':wx.NamedColour('SPRING GREEN'),
#	'1-st tablet':wx.NamedColour('gold'),
#	'pause':wx.NamedColour('light blue'),
#	'next 1-st tablet':wx.NamedColour('pink')}


def min_max(i):
    """Return length max and min of 6 last cycles
    from i item cycle.begin"""

    if len(cycle.begin) < 2 or i == 0:
        return cycle.period, cycle.period

    last_6 = []
    for k in range(i, 0, -1):
        span = (cycle.begin[k]-cycle.begin[k-1]+wx.TimeSpan.Hours(1)).GetDays()
        # wx.TimeSpan.Hours(1) - компенсация потери часа на летнем времени
        if 20 < span < 36:  # остальное в расчет не берем
            last_6.append(span)
            if len(last_6) >= 6:
                break

    if cycle.by_average and len(last_6) != 0:
        s = float(reduce(operator.add, last_6))  # sum of last_6
        cycle.period = int(round(s/len(last_6), 0))

    if last_6 == []:
        return cycle.period, cycle.period
    return min(last_6), max(last_6)


def calc_fert(year):
    """Рассчитывает фертильные периоды для year"""

    for k in list(cycle.mark.keys()):
        cycle.mark[k] = cycle.mark[k] & ~MARK_FERT &\
            ~MARK_OVUL & ~MARK_PROG & ~MARK_SAFESEX & ~MARK_BIRTH &\
            ~MARK_T22_28 & ~MARK_NEXT_TABLET

    # по прошлым циклам
    if cycle.begin == []:
        return
    year_b = wx.DateTime.FromDMY(1, 0, year)
    year_e = wx.DateTime.FromDMY(31, 11, year)
    for d in cycle.begin:
        i = cycle.begin.index(d)
        if i < len(cycle.begin)-1:
            if (cycle.begin[i+1]-cycle.begin[i]+wx.TimeSpan.Hours(1)).GetDays() < 21:
                # wx.TimeSpan.Hours(1) - потеря одного часа при переходе на
                # летнее время
                continue

        min, max = min_max(i)
        begin = d+wx.DateSpan.Days(min-18)  # begin fertile
        end = d+wx.DateSpan.Days(max-11)  # end fertile
        ovul = end-wx.DateSpan.Days(((max-11)-(min-18))/2)  # day of ovul
        if year_b <= ovul <= year_e:
            add_mark(ovul, MARK_OVUL, year)

        start = d+wx.DateSpan.Day()
        if i < len(cycle.begin)-1:
            last_cycle = (cycle.begin[i+1]-cycle.begin[i] +
                          wx.TimeSpan.Hours(1)).GetDays()
            if last_cycle > 35:
                stop = d+wx.DateSpan.Days(35)
            else:
                stop = cycle.begin[i+1]-wx.DateSpan.Day()
        else:
            stop = d+wx.DateSpan.Days(cycle.period-1)

        if (stop < year_b or start > year_e) and (d not in cycle.last):
            continue
        f = start
        while f.IsBetween(start, stop):
            if f.IsBetween(begin, end):
                add_mark(f, MARK_FERT, year)
            else:
                add_mark(f, MARK_SAFESEX, year)
            f = f+wx.DateSpan.Day()

        if d in cycle.last:  # calc birthday
            birth = d+wx.DateSpan.Days(280+cycle.period-28)
            if i < len(cycle.begin)-1:  # not last item
                if birth < cycle.begin[i+1]:
                    add_mark(birth, MARK_BIRTH, year)
            else:  # last item
                add_mark(birth, MARK_BIRTH, year)
                return

    # prognosis to future cycles
    cycle.prog_begin = []
    d = d+wx.DateSpan.Days(cycle.period)
    while d.GetYear() <= year:
        if cycle.tablet != [] and cycle.tablet[-1] <= d and \
                cycle.begin[-1] <= cycle.tablet[-1]:
            return
        if d.GetYear() == year:
            #	    cycle.prog_begin.append(d)
            add_mark(d, MARK_PROG, year)

        begin = d+wx.DateSpan.Days(min-18)  # начало периода
        end = d+wx.DateSpan.Days(max-11)  # конец периода
        ovul = end-wx.DateSpan.Days(((max-11)-(min-18))/2)  # day of ovul
        if year_b <= ovul <= year_e:
            add_mark(ovul, MARK_OVUL, year)

        start = d+wx.DateSpan.Day()
        stop = d+wx.DateSpan.Days(cycle.period-1)
        d = d+wx.DateSpan.Days(cycle.period)

        if stop < year_b or start > year_e:
            continue

        f = start
        while f.IsBetween(start, stop):
            if f.IsBetween(begin, end):
                add_mark(f, MARK_FERT, year)
            else:
                add_mark(f, MARK_SAFESEX, year)
            f = f+wx.DateSpan.Day()


def calc_tablet(year):
    """calculation result of using hormonal tablets"""

    if cycle.tablet == []:
        return
    year_b = wx.DateTime.FromDMY(1, 0, year)
    year_e = wx.DateTime.FromDMY(31, 11, year)
    for d in cycle.tablet:
        i = cycle.tablet.index(d)
        if i < len(cycle.tablet)-1:
            if (cycle.tablet[i+1]-cycle.tablet[i]+wx.TimeSpan.Hours(1)).GetDays() < 28:
                # incorrect using - must more 28 days
                continue
        for k in range(28):
            remove_mark(d+wx.DateSpan.Days(k), MARK_PROG | MARK_FERT |
                        MARK_NEXT_TABLET | MARK_OVUL | MARK_SAFESEX | MARK_BIRTH, year)
        for k in range(21, 28):
            add_mark(d+wx.DateSpan.Days(k), MARK_T22_28, year)
        add_mark(d+wx.DateSpan.Days(28), MARK_NEXT_TABLET, year)


def add_mark(date, mark, year):
    if date.GetYear() == year:
        k = date.GetDayOfYear()
        cycle.mark[k] = cycle.mark.get(k, 0) | mark


def remove_mark(date, mark, year):
    if date.GetYear() == year:
        k = date.GetDayOfYear()
        cycle.mark[k] = cycle.mark.get(k, 0) & ~mark


def is_set_mark(date, mark, year):
    if date.GetYear() == year:
        k = date.GetDayOfYear()
        return cycle.mark.get(k, 0) & mark
    else:
        return False


def reset_mark(year):
    cycle.mark.clear()
    for k in cycle.begin:
        if k.GetYear() == year:
            add_mark(k, MARK_BEGIN, year)
    for k in cycle.last:
        if k.GetYear() == year:
            add_mark(k, MARK_LAST, year)
    for k in cycle.tablet:
        if k.GetYear() == year:
            add_mark(k, MARK_TABLET, year)
    for k in list(cycle.note.keys()):
        if str(year) == k[0:4]:
            d = wx.DateTime.FromDMY(int(k[6:8]), int(k[4:6])-1, int(k[0:4]))
            add_mark(d, MARK_NOTE, year)


def info(day):
    """Возвращает строку информации по переданной дате."""

    s = day.Format('%d %B')
    if cycle.tablet != []:
        for d in cycle.tablet:
            if day.IsBetween(d, d+wx.DateSpan.Days(28)):
                t = (day-d+wx.TimeSpan.Hours(1)).GetDays()+1
                s += " - "
                if t <= 28:
                    s += _('tablet N ')+str(t)
                if 22 <= t <= 28:
                    s += _(' or pause')
                if t == 29:
                    s += _('next 1-st tablet')
                return s

    if cycle.begin == []:
        return s
    if day < cycle.begin[0]:
        return s  # день до начала первого цикла

    find = 0
    gestation = 0
    for d in cycle.begin:
        if day < d:
            find = 1
            break

    if find == 0:
        if d in cycle.last:
            gestation = 1
            d2 = d
        else:
            # ищем в будущих периодах
            while d <= day:
                if cycle.tablet != [] and cycle.tablet[-1] <= d and \
                        cycle.begin[-1] <= cycle.tablet[-1]:
                    return s
                d = d+wx.DateSpan.Days(cycle.period)
            find = 2

    # найдем следующее начало цикла
    if find == 1:
        i = cycle.begin.index(d)
        d2 = cycle.begin[i-1]
        if d2 in cycle.last:
            gestation = 1
    elif find == 2:
        d2 = d-wx.DateSpan.Days(cycle.period)

    if gestation:
        k = (day-d2+wx.TimeSpan.Hours(1)).GetDays()+1
        w = (k-1)/7
        s += " - "+str(k)+_('% day of gestation, ')+str(w)
        if w == 1:
            s += _('1 week')
        else:
            s += _('% weeks')
        s += ' + '+str(k-w*7)
        if (k-w*7) == 1:
            s += _('1 day')
        else:
            s += _('% days')
    else:
        p = (d-d2+wx.TimeSpan.Hours(1)).GetDays()
        k = (day-d2+wx.TimeSpan.Hours(1)).GetDays()+1

        d = d-wx.DateSpan.Day()
    s += " - "+_('%s day of period from %s to %s') % (str(k), d2.Format('%d %b'),
                                                      d.Format('%d %b')) + ' ' + _('length %s days') % (str(p))
    return s


# -------------------- Note --------------------
def add_note(date, txt):
    d = date.Format('%Y%m%d')
    cycle.note[d] = txt


def get_note(date):
    d = date.Format('%Y%m%d')
    return cycle.note.get(d, "")


def remove_note(date):
    d = date.Format('%Y%m%d')
    if d in cycle.note:
        del cycle.note[d]

# -------------------- Report --------------------


def report_year(year):
    if cycle.first_week_day == 0:
        calendar.setfirstweekday(calendar.MONDAY)
        days = list(range(1, 7)) + [0]
    else:
        calendar.setfirstweekday(calendar.SUNDAY)
        days = list(range(7))
    #sp=' '
    s = '<html><body><H3 align=center>%s</H3><pre>' % year
    dn = ''
    for i in days:
        dn += '%.2s ' % wx.DateTime_GetWeekDayName(i, wx.DateTime.Name_Abbr)
    dn = dn[:-1]
    dn = '%s  %s  %s<br>\n' % (dn, dn, dn)  # week days names
    for m in range(0, 12, 3):
        s += '<br>\n%s  %s  %s<br>\n' % (
            wx.DateTime_GetMonthName(m).center(20),
            wx.DateTime_GetMonthName(m+1).center(20),
            wx.DateTime_GetMonthName(m+2).center(20))
        s += dn
        data = []
        h = 0
        for k in range(m+1, m+4):
            cal = calendar.monthcalendar(year, k)
            data.append(calendar.monthcalendar(year, k))
            if h < len(cal):
                h = len(cal)
        for i in range(h):
            d_str = ''
            for n in range(3):
                for k in range(7):
                    if i < len(data[n]):
                        day_of_month = data[n][i][k]
                        if day_of_month:
                            d = wx.DateTime.FromDMY(day_of_month, m+n, year)
                            if is_set_mark(d, MARK_BEGIN, d.GetYear()):
                                d_str += '<u>%2s</u> ' % day_of_month
                            else:
                                d_str += '%2s ' % day_of_month
                        else:
                            d_str += '   '
                    else:
                        d_str += '   '
                d_str += ' '
            s += d_str[:-2] + '<br>\n'

    s += '</pre></body></html>'
    print(s)
    return s


def report_year_ical(year, fileobj):
    import socket
    hostname = socket.gethostname()

    def get_string(mark):
        if mark & MARK_LAST:
            return _("Conception")
        elif mark & MARK_BEGIN:
            return _("Beginning of cycle")
        elif mark & MARK_PROG:
            return _("Probable beginning of cycle")
        elif mark & MARK_TABLET:
            return _("1-st tablet")
        elif mark & MARK_OVUL:
            return _("Ovulation")
        elif mark & MARK_BIRTH:
            return _("Birth")
        else:
            return ""

    def make_event(description, mark, date):
        date2 = date + wx.TimeSpan.Days(1)
        datestr = "%04d%02d%02d" % (
            date.GetYear(), date.GetMonth() + 1, date.GetDay())
        datestr2 = "%04d%02d%02d" % (
            date2.GetYear(), date2.GetMonth() + 1, date2.GetDay())
        uid = "UID:cycle-%d-%sZ@%s" % (mark, datestr, hostname)
        return ["BEGIN:VEVENT", uid,
                "DTSTART;VALUE=DATE:" + datestr,
                "DTEND;VALUE=DATE:" + datestr2,
                "SUMMARY:" + description,
                "DESCRIPTION:" + description,
                "CLASS:PUBLIC",
                "END:VEVENT"]

    s = ["BEGIN:VCALENDAR",
         "CALSCALE:GREGORIAN",
         "PRODID:-//Cycle//NONSGML Cycle//EN",
         "VERSION:2.0"]

    days = list(cycle.mark.items())
    days.sort()
    for day, marks in days:
        if get_string(marks):
            d = wx.DateTime()
            d.SetYear(year)
            d.SetToYearDay(day)
            s.extend(make_event(get_string(marks), marks, d))

    s.append("END:VCALENDAR")

    print("\n".join(s), file=fileobj)


# -------------------- Add import --------------------
